#! /bin/sh
#------------------------------------------------------------------------------
# /var/install/config.d/postgresql16-update.sh - parameter update script
#
# Creation   :  2017-05-28 daniel
# Last Update:  2024-03-07 22:55:06
#
# Copyright (c) 2024 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#------------------------------------------------------------------------------

package='postgresql16'

#------------------------------------------------------------------------------
# include eisfair script library
#------------------------------------------------------------------------------

. /var/install/include/eislib
. /var/install/include/configlib


#------------------------------------------------------------------------------
# list of default values
#------------------------------------------------------------------------------

START_POSTGRESQL16='no'                      # Shall postgresql deamon be started

POSTGRESQL16_DATADIR='/srv/pgsql/16'         # location of database cluster
POSTGRESQL16_ENCODING='UTF8'                 # default database encoding

POSTGRESQL16_NETWORKING='yes'                # Enable network access to PostgreSQL
POSTGRESQL16_CONNECT_PORT='5432'             # TCP/IP port for network access
POSTGRESQL16_CONNECTIONS='10'                # max concurrent connects
POSTGRESQL16_ENABLE_SSL='no'                 # enable SSL connections
POSTGRESQL16_SSL_CERT_FILE='server.pem'      # name of server public key
POSTGRESQL16_SSL_KEY_FILE='server.key'       # name of server private key
POSTGRESQL16_SSL_CLIENT_VERIFY='no'          # enable client certificate verification
POSTGRESQL16_SSL_CA_FILE='trusted_ca.pem'    # name of a file containing trusted root CAs

POSTGRESQL16_HOST_N='2'                      # number of host entries

POSTGRESQL16_HOST_1_TYPE='local'             # Local or network(host) access?
POSTGRESQL16_HOST_1_NETWORK=''               # IP-Address of host (or network) 1
POSTGRESQL16_HOST_1_USERAUTH='trust'         # Authentication
POSTGRESQL16_HOST_1_DATABASE='all'           # affected databases
POSTGRESQL16_HOST_1_USER='all'               # affected users
POSTGRESQL16_HOST_1_CLIENTCERT='none'        # verification of client certificate

POSTGRESQL16_HOST_2_TYPE='host'              # Local or network(host) access?
POSTGRESQL16_HOST_2_NETWORK='127.0.0.1/32'   # IP-Address of host (or network) 1
POSTGRESQL16_HOST_2_USERAUTH='trust'         # Authentication
POSTGRESQL16_HOST_2_DATABASE='all'           # affected databases
POSTGRESQL16_HOST_2_USER='all'               # affected users
POSTGRESQL16_HOST_2_CLIENTCERT='none'        # verification of client certificate

POSTGRESQL16_AUTOVACUUM='yes'                # Start autovacuum process

# preformance options
POSTGRESQL16_MEMORY_LAYOUT='medium'          # layout: small, medium, large, huge
POSTGRESQL16_WRITE_MODE='normal'             # mode: secure, normal, fast, nosync

# advanced options (log options)
POSTGRESQL16_LOG_SETTINGS='no'
POSTGRESQL16_CLIENT_LOG_LEVEL='notice'      # log level for client programs
POSTGRESQL16_SERVER_LOG_LEVEL='notice'      # log level for server log
POSTGRESQL16_LOG_VERBOSE='no'               # verbose log messages
POSTGRESQL16_LOG_STATEMENTS='no'            # writes SQL statements to log


POSTGRESQL16_BACKUP_TIMER='*-*-10 01:00:00'  # schedule for autom. backup
POSTGRESQL16_BACKUP_TARGET='/srv/pgsql_backup'
                                             # target directory for backups
POSTGRESQL16_BACKUP_MOUNT=''                 # Command executed before backup
POSTGRESQL16_BACKUP_UMOUNT=''                # Command executed after backup
POSTGRESQL16_BACKUP_NOTIFY='root@localhost'  # E-mail address for error notification

POSTGRESQL16_BACKUP_CLUSTER='yes'            # backup complete database cluster
POSTGRESQL16_BACKUP_CLUSTER_USER='postgres'  # User to connect to db server
POSTGRESQL16_BACKUP_CLUSTER_MAX='7'          # number of db-cluster backup copies to keep

POSTGRESQL16_BACKUP_DATABASES='no'           # backup individual databases
POSTGRESQL16_BACKUP_N='0'                    # Number of databases to backup
POSTGRESQL16_BACKUP_1_DBNAME='template1'     # Name of database to backup
POSTGRESQL16_BACKUP_1_USER='postgres'        # User to connect to db server
POSTGRESQL16_BACKUP_1_MAX='7'                # Number of backup files to maintain


# -----------------------------------------------------------------------------
# rename old variables
# -----------------------------------------------------------------------------

rename_old_variables()
{
    # read old values
    if [ -f /etc/config.d/${package} ]
    then
        . /etc/config.d/${package}
    fi
}


# -----------------------------------------------------------------------------
# add new variables
# -----------------------------------------------------------------------------

add_new_variables()
{
    # no variables added at this time
    echo "" > /dev/null

    idx=1
    while [ "$idx" -le "$POSTGRESQL16_HOST_N" ]
    do
       eval clientcert='${POSTGRESQL16_HOST_'${idx}'_CLIENTCERT}'
       if [ -z "${clientcert}" ]
       then
           eval 'POSTGRESQL16_HOST_'${idx}'_CLIENTCERT'="none"
       fi
       idx=$(($idx + 1))
    done
}


# -----------------------------------------------------------------------------
# Write config and default files
# -----------------------------------------------------------------------------

write_config_file()
{
    local internal_conf_file=${1}
    (
    printgpl -conf "${package}" "2006-12-16" "dv" \
         "2024 the eisfair team, team(at)eisfair(dot)org"

    printgroup "PostgreSQL server settings"
    printvar "START_POSTGRESQL16" "Shall PostgreSQL be started"

    printgroup "PostgreSQL storage location"
    printvar "POSTGRESQL16_DATADIR" "location of database cluster"
    printvar "POSTGRESQL16_ENCODING" "default database encoding"

    printgroup "PostgreSQL network (TCP/IP) access"
    printvar "POSTGRESQL16_NETWORKING" "Enable network access to PostgreSQL"
    printvar "POSTGRESQL16_CONNECT_PORT" "TCP/IP port for network access"
    printvar "POSTGRESQL16_CONNECTIONS" "Max concurrent connects"
    printvar "POSTGRESQL16_ENABLE_SSL" "Enable SSL connections"
    printvar "POSTGRESQL16_SSL_CERT_FILE" "Name of server public key"
    printvar "POSTGRESQL16_SSL_KEY_FILE" "Name of server private key"
    printvar "POSTGRESQL16_SSL_CLIENT_VERIFY" "Enable client certificate verification"
    printvar "POSTGRESQL16_SSL_CA_FILE" "Name of file containing trusted CAs"

    printgroup "Host access table"
    printvar "POSTGRESQL16_HOST_N" "Number of host entries"
    echo ""
    printvar "POSTGRESQL16_HOST_1_TYPE" "Local or network access?"
    printvar "POSTGRESQL16_HOST_1_NETWORK" "IP-Address of host or network"
    printvar "POSTGRESQL16_HOST_1_USERAUTH" "Authentication"
    printvar "POSTGRESQL16_HOST_1_DATABASE" "Affected databases or 'all' for all"
    printvar "POSTGRESQL16_HOST_1_USER" "Affected users or 'all' for all"
    printvar "POSTGRESQL16_HOST_1_CLIENTCERT" "Verification of client certificate"
    echo ""
    printvar "POSTGRESQL16_HOST_2_TYPE" "Local or network access?"
    printvar "POSTGRESQL16_HOST_2_NETWORK" "IP-Address of host or network"
    printvar "POSTGRESQL16_HOST_2_USERAUTH" "Authentication"
    printvar "POSTGRESQL16_HOST_2_DATABASE" "Affected databases or 'all' for all"
    printvar "POSTGRESQL16_HOST_2_USER" "Affected users or 'all' for all"
    printvar "POSTGRESQL16_HOST_2_CLIENTCERT" "Verification of client certificate"

    idx=3
    while [ "$idx" -le "$POSTGRESQL16_HOST_N" ]
    do
       echo ""
       printvar "POSTGRESQL16_HOST_${idx}_TYPE" "Local or network access?"
       printvar "POSTGRESQL16_HOST_${idx}_NETWORK" "IP-Address of host or network"
       printvar "POSTGRESQL16_HOST_${idx}_USERAUTH" "Authentication"
       printvar "POSTGRESQL16_HOST_${idx}_DATABASE" "Affected databases or 'all' for all"
       printvar "POSTGRESQL16_HOST_${idx}_USER" "Affected users or 'all' for all"
       printvar "POSTGRESQL16_HOST_${idx}_CLIENTCERT" "Verification of client certificate"
       idx=$(($idx + 1))
    done

    printgroup "Performance options"
    printvar "POSTGRESQL16_AUTOVACUUM" "Start autovacuum process"
    printvar "POSTGRESQL16_MEMORY_LAYOUT" "Memory layout: small, medium, large, huge"
    printvar "POSTGRESQL16_WRITE_MODE" "Write mode: secure, normal, fast, nosync"

    printgroup "Log options (advanced)"
    printvar "POSTGRESQL16_LOG_SETTINGS" "manually overwrite default settings?"
    printvar "POSTGRESQL16_CLIENT_LOG_LEVEL" "Log level for client programs"
    printvar "POSTGRESQL16_SERVER_LOG_LEVEL" "log level for server log"
    printvar "POSTGRESQL16_LOG_VERBOSE" "Verbose log messages"
    printvar "POSTGRESQL16_LOG_STATEMENTS" "write SQL statements to log"

    printgroup "Database backup"
    printvar "POSTGRESQL16_BACKUP_TIMER" "Schedule for autom. backup"
    printvar "POSTGRESQL16_BACKUP_TARGET" "Target directory for backups"
    printvar "POSTGRESQL16_BACKUP_MOUNT" "Command executed before backup"
    printvar "POSTGRESQL16_BACKUP_UMOUNT" "Command executed after backup"
    printvar "POSTGRESQL16_BACKUP_NOTIFY" "E-mail address for error notification"
    echo ""

    printvar "POSTGRESQL16_BACKUP_CLUSTER" "Backup complete database cluster"
    printvar "POSTGRESQL16_BACKUP_CLUSTER_USER" "User to connect to db server"
    printvar "POSTGRESQL16_BACKUP_CLUSTER_MAX" "Number of db-cluster backup copies to keep"

    printvar "POSTGRESQL16_BACKUP_DATABASES" "backup individual databases"
    printvar "POSTGRESQL16_BACKUP_N" "Number of databases to backup"
    printvar "POSTGRESQL16_BACKUP_1_DBNAME" "Name of database to backup"
    printvar "POSTGRESQL16_BACKUP_1_USER" "User to connect to db server"
    printvar "POSTGRESQL16_BACKUP_1_MAX" "Number of backup files to maintain"
    idx=2
    while [ "$idx" -le "$POSTGRESQL16_BACKUP_N" ]
    do
       echo ""
       printvar "POSTGRESQL16_BACKUP_${idx}_DBNAME" "Name of database to backup"
       printvar "POSTGRESQL16_BACKUP_${idx}_USER" "User to connect to db server"
       printvar "POSTGRESQL16_BACKUP_${idx}_MAX" "Number of backup files to maintain"
       idx=$(($idx + 1))
    done
    echo ""

    printend
    ) > ${internal_conf_file}
    # Set rights
    chmod 0644 ${internal_conf_file}
    chown root ${internal_conf_file}
}


# ---------------------------------------------------------------------------
# Create the check.d file
# ---------------------------------------------------------------------------

write_check_file()
{
    (
    printgpl -check "${package}" "2007-03-03" "dv" \
         "2024 the eisfair team, team(at)eisfair(dot)org"
    echo "START_POSTGRESQL16                -                       -                   YESNO"
    echo "POSTGRESQL16_NETWORKING           START_POSTGRESQL16      -                   YESNO"
    echo "POSTGRESQL16_DATADIR              START_POSTGRESQL16      -                   ABS_PATH"
    echo "POSTGRESQL16_ENCODING             START_POSTGRESQL16      -                   PG16_ENCODING"
    echo "POSTGRESQL16_CONNECT_PORT         START_POSTGRESQL16      -                   PORT"
    echo "POSTGRESQL16_CONNECT_PORT         START_POSTGRESQL16      -                   PGSQL_COMMON_PORT_CUI"
    echo "POSTGRESQL16_CONNECTIONS          START_POSTGRESQL16      -                   NUMERIC"
    echo "POSTGRESQL16_CONNECTIONS          START_POSTGRESQL16      -                   PG16_MAXCONN"
    echo "POSTGRESQL16_ENABLE_SSL           START_POSTGRESQL16      -                   YESNO"
    echo "POSTGRESQL16_SSL_CERT_FILE        POSTGRESQL16_ENABLE_SSL -                   PG16_SSL_CERTS_CUI"
    echo "POSTGRESQL16_SSL_KEY_FILE         POSTGRESQL16_ENABLE_SSL -                   PG16_SSL_KEYS_CUI"
    echo "POSTGRESQL16_SSL_CLIENT_VERIFY    POSTGRESQL16_ENABLE_SSL -                   YESNO"
    echo "POSTGRESQL16_SSL_CA_FILE          POSTGRESQL16_SSL_CLIENT_VERIFY -            PG16_SSL_CERTS_CUI"
    echo "POSTGRESQL16_HOST_N               START_POSTGRESQL16      -                   NUMERIC"
    echo "POSTGRESQL16_HOST_%_TYPE          START_POSTGRESQL16      POSTGRESQL16_HOST_N   PG16_ACCESSTYPE"
    echo "POSTGRESQL16_HOST_%_NETWORK       START_POSTGRESQL16      POSTGRESQL16_HOST_N   PG16_NETWORK"
    echo "POSTGRESQL16_HOST_%_USERAUTH      START_POSTGRESQL16      POSTGRESQL16_HOST_N   PG16_AUTHENTICATION"
    echo "POSTGRESQL16_HOST_%_DATABASE      START_POSTGRESQL16      POSTGRESQL16_HOST_N   NOTEMPTY"
    echo "POSTGRESQL16_HOST_%_USER          START_POSTGRESQL16      POSTGRESQL16_HOST_N   NOTEMPTY"
    echo "POSTGRESQL16_HOST_%_CLIENTCERT    START_POSTGRESQL16      POSTGRESQL16_HOST_N   PG16_CLIENTCERT"

    echo "POSTGRESQL16_AUTOVACUUM           START_POSTGRESQL16      -                   YESNO"
    echo "POSTGRESQL16_MEMORY_LAYOUT        START_POSTGRESQL16      -                   PG16_MEM_LAYOUT"
    echo "POSTGRESQL16_WRITE_MODE           START_POSTGRESQL16      -                   PG16_WRITE_MODE"

    echo "POSTGRESQL16_LOG_SETTINGS         START_POSTGRESQL16            -             YESNO"
    echo "POSTGRESQL16_CLIENT_LOG_LEVEL     POSTGRESQL16_LOG_SETTINGS     -             PG16_LOGLEVEL"
    echo "POSTGRESQL16_SERVER_LOG_LEVEL     POSTGRESQL16_LOG_SETTINGS     -             PG16_LOGLEVEL"
    echo "POSTGRESQL16_LOG_VERBOSE          POSTGRESQL16_LOG_SETTINGS     -             YESNO"
    echo "POSTGRESQL16_LOG_STATEMENTS       POSTGRESQL16_LOG_SETTINGS     -             YESNO"

    echo "POSTGRESQL16_BACKUP_TIMER         START_POSTGRESQL16      -                   ON_CALENDAR"
    echo "POSTGRESQL16_BACKUP_TARGET        START_POSTGRESQL16      -                   ABS_PATH"
    echo "POSTGRESQL16_BACKUP_MOUNT         START_POSTGRESQL16      -                   NONE"
    echo "POSTGRESQL16_BACKUP_UMOUNT        START_POSTGRESQL16      -                   NONE"
    echo "POSTGRESQL16_BACKUP_NOTIFY        START_POSTGRESQL16      -                   EMAILADDR"

    echo "POSTGRESQL16_BACKUP_CLUSTER       START_POSTGRESQL16                 -                   YESNO"
    echo "POSTGRESQL16_BACKUP_CLUSTER_USER  POSTGRESQL16_BACKUP_CLUSTER        -                   NOTEMPTY"
    echo "POSTGRESQL16_BACKUP_CLUSTER_MAX   POSTGRESQL16_BACKUP_CLUSTER        -                   NUMERIC"

    echo "POSTGRESQL16_BACKUP_DATABASES     START_POSTGRESQL16                 -                   YESNO"
    echo "POSTGRESQL16_BACKUP_N             POSTGRESQL16_BACKUP_DATABASES      -                   NUMERIC"
    echo "POSTGRESQL16_BACKUP_%_DBNAME      POSTGRESQL16_BACKUP_DATABASES      POSTGRESQL16_BACKUP_N NOTEMPTY"
    echo "POSTGRESQL16_BACKUP_%_USER        POSTGRESQL16_BACKUP_DATABASES      POSTGRESQL16_BACKUP_N NOTEMPTY"
    echo "POSTGRESQL16_BACKUP_%_MAX         POSTGRESQL16_BACKUP_DATABASES      POSTGRESQL16_BACKUP_N NUMERIC"

    echo ""
    ) > /etc/check.d/${package}
    # Set rights for check.d file
    chmod 0644 /etc/check.d/${package}
    chown root /etc/check.d/${package}

    (
    printgpl -check_exp "${package}" "2017-04-26" "dv" \
         "2024 the eisfair team, team(at)eisfair(dot)org"
    echo "PG16_ENCODING           = 'BIG5|EUC_CN|EUC_JP|EUC_JIS_2004|EUC_KR|EUC_TW|GB18030|GBK|ISO_8859_5|ISO_8859_6|ISO_8859_7|ISO_8859_8|JOHAB|KOI8R|KOI8U|LATIN1|LATIN2|LATIN3|LATIN4|LATIN5|LATIN6|LATIN7|LATIN8|LATIN9|LATIN10|MULE_INTERNAL|SJIS|SHIFT_JIS_2004|SQL_ASCII|UHC|UTF8|WIN866|WIN874|WIN1250|WIN1251|WIN1252|WIN1253|WIN1254|WIN1255|WIN1256|WIN1257|WIN1258'"
    echo "                        : 'Wrong encoding. Read documentation!'"
    echo "PG16_ACCESSTYPE         = 'local|host|hostssl|hostnossl'"
    echo "                        : 'Wrong access type! Only \"local\", \"host\", \"hostssl\" or"
    echo "                           \"hostnossl\" are allowed'"
    echo "PG16_NETWORK            = '()|(RE:NETWORK)'"
    echo "                        : 'no valid network specification, should be a network"
    echo "                           address followed by a netmask, for instance 192.168.6.0/24'"
    echo "PG16_SSL_CERTS_CUI      = '()|(RE:NOTEMPTY)'"
    echo "                        : 'SSL certificate name expected'"
    echo "PG16_SSL_KEYS_CUI       = '()|(RE:NOTEMPTY)'"
    echo "                        : 'SSL private key name expected'"
    echo "PG16_AUTHENTICATION     = 'trust|reject|md5|crypt|password|krb4|krb5|ident|peer|pam'"
    echo "                        : 'Wrong authentication method! Only \"trust\", \"reject\","
    echo "                           \"md5\", \"crypt\", \"password\", \"krb4\", \"krb5\","
    echo "                           \"ident\", \"peer\" or \"pam\" are allowed.'"
    echo "PG16_CLIENTCERT         = 'none|verify-ca|verify-full'"
    echo "                        : 'Wrong certificate verification! Only \"none\", \"verify-ca\" or"
    echo "                           \"verify-full\" are allowed.'"
    echo "PG16_MEM_LAYOUT         = 'small|medium|large|huge'"
    echo "                        : 'Wrong memory layout! Only \"small\", \"medium\", \"large\" or \"huge\".'"
    echo "PG16_WRITE_MODE         = 'secure|normal|fast|nosync'"
    echo "                        : 'Wrong write mode! Only \"secure\", \"normal\", \"fast\" or \"nosync\".'"
    echo "PG16_LOGLEVEL           = 'debug5|debug4|debug3|debug2|debug1|log|notice|warning|error|fatal|panic'"
    echo "                        : 'Wrong loglevel! Only \"debug5\", \"debug4\", \"debug3\","
    echo "                           \"debug2\",\"debug1\", \"log\", \"notice\", \"warning\","
    echo "                           \"error\", \"fatal\", \"panic\" are allowed.'"
    echo "PG16_BYTENUM            = '(RE:NUMERIC)(GB|MB|kB|())'"
    echo "                        : 'Wrong memory size given! Use number with optional postfix "
    echo "                           \"GB\", \"MB\" or \"kB\".'"
    echo "PG16_MAXCONN            = '[1-9][0-9]{0,2}|[1][0-9]{3}|2000'"
    echo "                        : 'value must be in range 1..2000'"
    echo "PGSQL_COMMON_PORT_CUI = '(RE:PORT)'"
    echo "                        : 'enter a valid network port in the range between 1 and 65535'"
    echo ""
    ) > /etc/check.d/${package}.exp
    # Set rights for check.exp file
    chmod 0644 /etc/check.d/${package}.exp
    chown root /etc/check.d/${package}.exp
}


# -----------------------------------------------------------------------------
# Main
# -----------------------------------------------------------------------------

# write default config file
write_config_file /etc/default.d/${package}

# update from old version
rename_old_variables
add_new_variables

# write new config file
write_config_file /etc/config.d/${package}

# write check.d file
write_check_file


